function SBVAR = SetupSBVAR_Proposal(n_lags, nu, Phi, Psi, Omega_inv, nu_bar, Phi_bar, Psi_bar, Omega_bar_inv, Y, X, U, V)
%
% The fields available are:
%
%  prior   - 'Conjugate'
%  n_lags  - number of lags
%  ntilde  - number endogenous variables
%  mtilde  - number predetermined variables
%  T       - number observations
%  Y       - endogenous data
%  X       - predetermined data
%  YY      - Y'*Y
%  XX      - X'*X
%  XY      - X'*Y
%  nu_bar            - generalized Gaussian prior parameter
%  Phi_bar_inv       - generalized Gaussian prior parameter
%  mu_bar            - matrix Gaussian prior parameter
%  Omega_bar_inv     - matrix Gaussian prior parameter
%  nu_tilde          - generalized Gaussian posterior parameter
%  Phi_tilde_inv     - generalized Gaussian posterior parameter
%  mu_tilde          - matrix Gaussian posterior parameter
%  Omega_tilde_inv   - matrix Gaussian posterior parameter
%  U     Llambda_0*e{j}    = U{j}*gamma_0{j}
%  V     Llambda_plus*e{j} = V{j}*gamma_+{j} 


% prior type
SBVAR.prior = 'Conjugate';

% sizes
SBVAR.n_lags = n_lags;
SBVAR.ntilde = size(Y,2);
SBVAR.mtilde = size(X,2);
SBVAR.T      = size(Y,1);


% unrestricted prior
SBVAR.nu        = nu;
SBVAR.Phi       = Phi;
SBVAR.Psi       = Psi;
SBVAR.Omega_inv = Omega_inv;

% unrestricted posterior
[SBVAR.nu_tilde, SBVAR.Phi_tilde, SBVAR.Psi_tilde, SBVAR.Omega_tilde_inv] = SBVAR_PosteriorParameters(SBVAR.nu, SBVAR.Phi, SBVAR.Psi, SBVAR.Omega_inv, Y, X);


% unrestricted proposal prior
SBVAR.nu_bar        = nu_bar;
SBVAR.Phi_bar       = Phi_bar;
SBVAR.Psi_bar       = Psi_bar;
SBVAR.Omega_bar_inv = Omega_bar_inv;

% unrestricted proposal posterior 
[SBVAR.nu_hat, SBVAR.Phi_hat, SBVAR.Psi_hat, SBVAR.Omega_hat_inv] = SBVAR_PosteriorParameters(SBVAR.nu_bar, SBVAR.Phi_bar, SBVAR.Psi_bar, SBVAR.Omega_bar_inv, Y, X);

% restrictions
SBVAR.U = U;
SBVAR.V = V;


% restricted posterior proposal
SBVAR.nuhat      = SBVAR.T + nu_bar;
SBVAR.Hjhat      = cell(SBVAR.ntilde,1);
SBVAR.Pjhat      = cell(SBVAR.ntilde,1);
SBVAR.Sjhat      = cell(SBVAR.ntilde,1);
SBVAR.UThat      = cell(SBVAR.ntilde,1);



for j=1:SBVAR.ntilde
    
    Omega_hat_inv_j  = SBVAR.Omega_hat_inv(1+(j-1)*SBVAR.mtilde:j*SBVAR.mtilde,1+(j-1)*SBVAR.mtilde:j*SBVAR.mtilde);
    
    Psi_hat_j        = SBVAR.Psi_hat(1+(j-1)*SBVAR.mtilde:j*SBVAR.mtilde,1+(j-1)*SBVAR.ntilde:j*SBVAR.ntilde);
    
    Phi_hat_j        = SBVAR.Phi_hat(1+(j-1)*SBVAR.ntilde:j*SBVAR.ntilde,1+(j-1)*SBVAR.ntilde:j*SBVAR.ntilde);
  
    SBVAR.Hjhat{j}   = (V{j}'*Omega_hat_inv_j*V{j})\eye(size(V{j},2));
    SBVAR.Pjhat{j}   = SBVAR.Hjhat{j}*(V{j}'*Omega_hat_inv_j*Psi_hat_j*U{j}); 
    SBVAR.Sjhat{j}   = (U{j}'*Phi_hat_j*U{j} + U{j}'*Psi_hat_j'*Omega_hat_inv_j*Psi_hat_j*U{j} - SBVAR.Pjhat{j}'*(SBVAR.Hjhat{j}\SBVAR.Pjhat{j}))\eye(size(U{j},2));

    % The following two lines are necessary to draw from the Generalized
    % Normal described in Appendix A.4. See Waggoner and Zha (2003) for
    % details.

    SBVAR.That       = chol(0.5*(SBVAR.Sjhat{j}+SBVAR.Sjhat{j}'))'; 
    SBVAR.UThat{j}   = U{j}*SBVAR.That;   


end

