function [Y_bar, X_bar] = SimsZhaDummyObservations(Y, X, n_lags, periods_per_year, hyperparameters, variance_scale)
%
% Y - endogenous data, each row is an observation
% X - predetermined data, each row is an observation
% n_lags - number of lags
% periods_per_year - number periods per year (monthly = 12, quarterly = 4) 
% hyperparameters - parameters controlling prior
% variance_scale - optional scaling parameter
%
% Implements the Sims/Zha prior with the expanded hyperparameter set.  The
% hyperparametes are:
%
% mu(1) - controls overall tightness of prior except for
%          sum-of-coefficients and co-persistence priors
% mu(2) - additional tightness on random walk prior
% mu(3) - additional tightness on constant term
% mu(4) - (new) controls shape of lag suppression - set to one to have old
%          prior
% mu(5) - suppresses long lags
% mu(6) - sum-of-coefficients prior
% mu(7) - co-persistence prior
%

if numel(hyperparameters) ~= 7
    error('not enough hyperparameters');
end

n_vars=size(Y,2);
n_predetermined=size(X,2);
if n_predetermined ~= n_vars*n_lags+1
    error('number predetermined not equal to number variables times number lags plus a constant');
end

% standard deviation of univariate AR residuals
n_obs=size(Y,2);
s=zeros(n_vars,1);
for ii=1:n_vars
    select=[ii:n_vars:n_predetermined-1 n_predetermined];
    x=X(:,select);
    [Q,~]=qr(x,0);
    y=Y(:,ii);
    e=y - Q*(Q'*y);
    s(ii)=sqrt(dot(e,e)/n_obs);
end

% dummy observations
%   e(j,m) is the jth row of mxm identity matrix
Y_bar=zeros(2+n_vars*(n_lags+2),n_vars);
X_bar=zeros(2+n_vars*(n_lags+2),n_predetermined);

% prior on a(k,i): dummy observations of the form
%   y=s(i)/mu(1) * e(i,n_vars)
%   x=0
start_row=0;
for ii=1:n_vars
    Y_bar(start_row+ii,ii)=s(ii)/hyperparameters(1);
end
start_row=start_row+n_vars;

% prior on constant: dummy observation of the form
%   y=0
%   x=1.0/(mu(1)*mu(3)) * e(n_pred,n_predetermined)
X_bar(start_row+1,n_predetermined)=1.0/(hyperparameters(1)*hyperparameters(3));
start_row=start_row+1;

if n_lags > 0
    % mean of initial data contained in the first observations of the predetermined data
    m=mean(reshape(X(1,1:n_vars*n_lags),n_vars,n_lags),2);

    % random walk prior: dummy observations of the form
    %   y=s(i)/(mu(2)*mu(1)) * e(i,n_vars)
    %   x=s(i)/(mu(2)*mu(1)) * e(i,n_predetermined)
    for ii=1:n_vars
        Y_bar(start_row+ii,ii)=s(ii)/(hyperparameters(1)*hyperparameters(2));
        X_bar(start_row+ii,ii)=s(ii)/(hyperparameters(1)*hyperparameters(2));
    end
    start_row=start_row+n_vars;
    
    % lag decay prior: dummy observations of the form
    %   y=0
    %   x=s(i)*(4*mu(4)*(j-1)/periods_per_year+1)^mu(5)/(mu(1)*mu(2)) * e((j-1)*n_vars+i,n_predetermined)
    for jj=2:n_lags
        for ii=1:n_vars
            X_bar(start_row+ii,(jj-1)*n_vars+ii)=s(ii)*(4.0*hyperparameters(4)*(jj-1)/periods_per_year+1.0)^hyperparameters(5) ...
                /(hyperparameters(1)*hyperparameters(2));
        end
        start_row=start_row+n_vars;
    end
    
    % sums-of-coefficients prior: dummy observations of the form
    %   y=mu(6)*m(i) * e(i,n_vars)
    %   x=sum_j { mu(6)*m(i) * e((j-1)*n_vars+i,n_predetermined) }
    for ii=1:n_vars
        Y_bar(start_row+ii,ii)=hyperparameters(6)*m(ii);
        for jj=1:n_lags
            X_bar(start_row+ii,(jj-1)*n_vars+ii)=hyperparameters(6)*m(ii);
        end
    end
    start_row=start_row+n_vars;
    
    % co-persistence prior: dummy observation of the form
    %   y=sum_i { mu(7)*m(i) * e(i,n_vars) }
    %   x=sum_i,j { mu(7)*m(i) * e((j-1)*n_vars+i,n_predetermined) } + mu(7) * e(n_pred,n_pred)
    for ii=1:n_vars
        Y_bar(start_row+1,ii)=hyperparameters(7)*m(ii);
        for jj=1:n_lags
            X_bar(start_row+1,(jj-1)*n_vars+ii)=hyperparameters(7)*m(ii);
        end
    end
    X_bar(start_row+1,n_predetermined)=hyperparameters(7);
end

if nargin == 6
    Y_bar=Y_bar/sqrt(variance_scale);
    X_bar=X_bar/sqrt(variance_scale);
end

end     % function GetDummyObservations
