function draw = RestrictedGeneralizedGaussianDraw(A0, idx, nu, UT, burn_in)
%
% A0: n x n matrix
%
% idx: 1 <= idx <= n
%
% nu: nu >= n
%
% UT: n-dimensional cell containing n x k_j matrices with k_j <= n.  UT can
% obtained from a call to SetupRestrictionsGaussianGeneralizedGaussian().
%
% burn_in: optional parameter, default value is 10
%
% Draws the first idx columns of A0 from the restricted generalized
% gaussian distribution, conditional on the last n-idx columns of A0.


% default value for burn_in
if (nargin < 5) || (burn_in < 1)
    burn_in=10;
end

% initial value of A0
n=numel(UT);
draw = A0;
for k=1:idx
    draw(:,k)=UT{k}*randn(size(UT{k},2),1);
end

for count=1:burn_in
    for k=1:idx
        % w is a non-zero vector that is perpendicular to all columns of A0
        % except the kth
        % w1 = T'(k)*U'(k)*w / ||T'(k)*U'(k)*w||
        w1=UT{k}'*GetNullVector(draw,k);
        w1=(1.0/norm(w1))*w1;
        
        % n dimension w1
        d=numel(w1);
        
        % Draw univariate generalised Gaussian component
        b=UnivariateGeneralizedGaussian(nu-n+1,1)*w1; 
        
        %===== Draw Gaussian components =====%
        
        % Find largest element of w1
        m=1;
        scale=abs(w1(m));
        for i=2:d
            if abs(w1(i)) > scale
                m=i;
                scale=abs(w1(m));
            end
        end
        c0=scale^2;
        
        % j < m
        % c0 = w1[m]^2 + w1[0]^2 + ... + w1[j-1]^2
        % c1 = w1[m]^2 + w1[0]^2 + ... + w1[j]^2
        % wj = (w1[0]*w1[j] ... w1[j-1]*w1[j] -c0 ... w1[m]*w1[j] ...)/sqrt(c0*c1)
        for j=1:m-1
            c1 = c0 + w1(j)^2;
            scale = randn/sqrt(c0*c1);
            b(j) = b(j) - scale*c0;
            scale = scale*w1(j);
            b(1:j) = b(1:j) + scale*w1(1:j);
            b(m)=b(m) + scale*w1(m);
            c0=c1;
        end
        
        % j > m
        % c0 = w1[0]^2 + ... + w1[j-1]^2
        % c1 = w1[0]^2 + ... + w1[j]^2
        % w_j = (w1[0]*w1[j] ... w1[j-1]*w1[j] -c0 ...)/sqrt(c0*c1)
        for j=m+1:d
            c1 = c0 + w1(j)^2;
            scale = randn/sqrt(c0*c1);
            b(j) = b(j) - scale*c0;
            scale = scale*w1(j);
            b(1:j) = b(1:j) + scale*w1(1:j);
            c0=c1;
        end   
        
        % A0(:,k) = U[k]*T(k)*b
	    draw(:,k) = UT{k}*b;

    end
end
